﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Threading.Tasks;

namespace TxtFileCutApart
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length < 3)
            {
                Console.WriteLine("缺少参数。");
                return;
            }
            if (!File.Exists(args[0]))
            {
                Console.WriteLine("分割文件不存在");
                return;
            }
            if (!Directory.Exists(args[1]))
            {
                Console.WriteLine("分割目录不存在");
                return;
            }
            long size = Convert.ToInt64(args[2]);
            List<string> types = new List<string>() {"size","count","row" };
            Dictionary<string, TxtFileCutApart> dicFileCut = new Dictionary<string, TxtFileCutApart>() { {"size",new TxtFileCutApartFileSize() }, { "count", new TxtFileCutApartFileCount() } , { "row", new TxtFileCutApartRows() } };
            string type =types[0];
            if (args.Length == 4&&types.Contains(args[3]))
            {
                type =args[3];
            }
            Console.WriteLine("开始分割，请稍候……");
            Stopwatch st = new Stopwatch();
            TxtFileCutApart txtFileCut = dicFileCut[type];
            st.Start();
            txtFileCut.FileCut(args[0],args[1],size);
            st.Stop();
            Console.WriteLine($"完成,耗时:{st.ElapsedMilliseconds/1000}s");
        }
    }

    public abstract class TxtFileCutApart
    {
        public abstract void FileCut(string sourcePath, string targetFolder,long size);
    }
    public  class TxtFileCutApartFileCount : TxtFileCutApart
    {
        public override void FileCut(string sourcePath, string targetFolder, long fileCount)
        {
            if (fileCount <= 0)
            {
                return;
            }
            FileInfo fileInfo = new FileInfo(sourcePath);
            string fileName = fileInfo.Name.Replace(fileInfo.Extension, "");
            long fileLength = fileInfo.Length;
            FileStream fsRead = new FileStream(sourcePath, FileMode.Open, FileAccess.Read, FileShare.Read);
            BinaryReader br = new BinaryReader(fsRead);
            int defaultBurrerLength = 1024 * 1024;
            long fileSizeLength = Convert.ToInt64((Math.Ceiling(fileLength / Convert.ToDouble(fileCount))));
            byte[] buffer = new byte[defaultBurrerLength];
            int readLength = 0;
            int fileIndex = 1;
            long readFileLength = 0;
            while (readFileLength < fileLength)
            {
                string writeFile = Path.Combine(targetFolder, $"{fileName}_{fileIndex}{fileInfo.Extension}");
                FileStream fsWrite = new FileStream(writeFile, FileMode.CreateNew, FileAccess.Write);
                BinaryWriter bw = new BinaryWriter(fsWrite);
                long singleFileLength = 0;
                while ((readLength = br.Read(buffer, 0, buffer.Length)) > 0)
                {
                    bw.Write(buffer, 0, readLength);
                    readFileLength += readLength;
                    singleFileLength += readLength;
                    if (singleFileLength >= fileSizeLength)
                    {
                        bw?.Close();
                        bw?.Dispose();
                        fsWrite?.Close();
                        fsWrite?.Dispose();
                        break;
                    }
                }
                bw?.Close();
                bw?.Dispose();
                fsWrite?.Close();
                fsWrite?.Dispose();
                fileIndex++;
            }
            br?.Close();
            br?.Dispose();
            fsRead?.Close();
            fsRead.Dispose();
        }
    }
    public  class TxtFileCutApartFileSize : TxtFileCutApart
    {
        public override void FileCut(string sourcePath, string targetFolder, long fileSize)
        {
            if (fileSize <= 0)
            {
                return;
            }
            FileInfo fileInfo = new FileInfo(sourcePath);
            string fileName = fileInfo.Name.Replace(fileInfo.Extension, "");

            FileStream fsRead = new FileStream(sourcePath, FileMode.Open, FileAccess.Read, FileShare.Read);
            BinaryReader br = new BinaryReader(fsRead);
            int defaultBurrerLength = 1024 * 1024;
            long fileSizeLength = fileSize * defaultBurrerLength;
            byte[] buffer = new byte[defaultBurrerLength];
            int readLength = 0;
            int fileIndex = 1;
            long fileLength = fileInfo.Length;
            long readFileLength = 0;
            while (readFileLength < fileLength)
            {
                string writeFile = Path.Combine(targetFolder, $"{fileName}_{fileIndex}{fileInfo.Extension}");
                FileStream fsWrite = new FileStream(writeFile, FileMode.CreateNew, FileAccess.Write);
                BinaryWriter bw = new BinaryWriter(fsWrite);
                long singleFileLength = 0;
                while ((readLength = br.Read(buffer, 0, buffer.Length)) > 0)
                {
                    bw.Write(buffer, 0, readLength);
                    readFileLength += readLength;
                    singleFileLength += readLength;
                    if (singleFileLength >= fileSizeLength)
                    {
                        bw?.Close();
                        bw?.Dispose();
                        fsWrite?.Close();
                        fsWrite?.Dispose();
                        break;
                    }
                }
                bw?.Close();
                bw?.Dispose();
                fsWrite?.Close();
                fsWrite?.Dispose();
                fileIndex++;
            }
            br?.Close();
            br?.Dispose();
            fsRead?.Close();
            fsRead.Dispose();
        }
    }
    public  class TxtFileCutApartRows: TxtFileCutApart
    {
        public override void FileCut(string sourcePath, string targetFolder, long row)
        {
            if (row <= 0)
            {
                return;
            }
            FileInfo fileInfo = new FileInfo(sourcePath);
            string fileName = fileInfo.Name.Replace(fileInfo.Extension, "");
            string lineData = "";
            StreamReader sr = new StreamReader(sourcePath);
            StreamWriter sw = null;

            long readEowIndex = 1, index = 1;
            while ((lineData = sr.ReadLine()) != null)
            {
                if (sw == null || readEowIndex >= row)
                {
                    sw?.Close();
                    sw?.Dispose();
                    string _fileName = Path.Combine(targetFolder, $"{fileName}_{index}{fileInfo.Extension}");
                    sw = new StreamWriter(_fileName);
                    readEowIndex = 1;
                    index++;
                }
                if (row >= readEowIndex)
                {
                    sw.WriteLine(lineData);
                    sw.Flush();
                    readEowIndex++;
                }
            }
            sw?.Close();
            sw?.Dispose();
            sr?.Close();
            sr.Dispose();
        }
    }
}
